import UIKit
import CoreLocation
import CoreBluetooth

//ビーコンマネージャ(7)
class BeaconManager: NSObject,
    CBCentralManagerDelegate,
    CLLocationManagerDelegate {
    //システム
    var _centralManager: CBCentralManager!   //セントラルマネージャ
    var _locationManager: CLLocationManager! //ロケーションマネージャ
    var _delegate: BeaconRegionDelegate!     //デリゲート

    //情報
    var _regions = [BeaconRegion]() //領域群
    
    
//====================
//初期化
//====================
    //初期化(8)
    init(delegate: BeaconRegionDelegate?) {
        super.init()
        _delegate = delegate
        
        //セントラルマネージャの生成
        let options = [CBCentralManagerOptionShowPowerAlertKey: NSNumber(bool: true)]
        _centralManager = CBCentralManager(delegate: self, queue: nil, options: options)
        
        //ロケーションマネージャの生成
        _locationManager = CLLocationManager()
        _locationManager.delegate = self
        _locationManager.requestAlwaysAuthorization()
    }
    
    //リージョンの追加(9)
    func addRegion(uuid: String) -> BeaconRegion {
        let region = BeaconRegion(locationManager: _locationManager,
            delegate: _delegate, uuid: uuid)
        _regions.append(region)
        return region
    }
    
    //リージョンの削除(10)
    func clearRegions() {
        stopMonitoring()
        _regions.removeAll()
    }
    
    //リージョン観測の開始(11)
    func startMonitoring() {
        for (beacon) in _regions {
            beacon.startMonitoring()
        }
    }
    
    //リージョン観測の停止(12)
    func stopMonitoring() {
        for (beacon) in _regions {
            beacon.stopMonitoring()
        }
    }
    
    
//====================
//CBCentralManagerDelegate(13)
//====================
    //CBCentralManagerのステータスが更新されるとコールされる
    func centralManagerDidUpdateState(central: CBCentralManager) {
        if central.state == CBCentralManagerState.PoweredOn {
            startMonitoring()
        }
    }
    
    
//====================
//CLLocationManagerDelegate(14)
//====================
    //認証状態変更時に呼ばれる
    func locationManager(manager: CLLocationManager,
        didChangeAuthorizationStatus status: CLAuthorizationStatus) {
        if status == CLAuthorizationStatus.AuthorizedAlways ||
            status == CLAuthorizationStatus.AuthorizedWhenInUse {
            startMonitoring()
        }
    }
    
    //リージョン観測の開始時に呼ばれる
    func locationManager(manager: CLLocationManager,
        didStartMonitoringForRegion region: CLRegion) {
        for (beacon) in _regions {
            if beacon._identifier == region.identifier {
                beacon.locationManager(manager,
                    didStartMonitoringForRegion: region)
                break
            }
        }
    }
    
    //リージョン状態の通知時に呼ばれる
    func locationManager(manager: CLLocationManager,
        didDetermineState state: CLRegionState, forRegion region: CLRegion) {
        for (beacon) in _regions {
            if beacon._identifier == region.identifier {
                beacon.locationManager(manager,
                    didDetermineState: state, forRegion: region)
                break
            }
        }
    }
    
    //リージョン外からリージョン内に移動した時に呼ばれる
    //(開始時にリージョン内にいると呼ばれない)
    func locationManager(manager: CLLocationManager,
        didEnterRegion region: CLRegion) {
        for (beacon) in _regions {
            if beacon._identifier == region.identifier {
                beacon.locationManager(manager, didEnterRegion: region)
                break
            }
        }
    }
    
    //リージョン内からリージョン外に移動した時に呼ばれる
    //(20秒〜1分かかる)
    func locationManager(manager: CLLocationManager,
        didExitRegion region: CLRegion) {
        for (beacon) in _regions {
            if beacon._identifier == region.identifier {
                beacon.locationManager(manager, didExitRegion: region)
                break
            }
        }
    }
    
    //リージョン内で距離が変化した時に呼ばれる(1秒毎に呼ばれる)
    func locationManager(manager: CLLocationManager,
        didRangeBeacons beacons: [CLBeacon],
        inRegion region: CLBeaconRegion) {
        for (beacon) in _regions {
            if beacon._identifier == region.identifier {
                beacon.locationManager(manager,
                    didRangeBeacons: beacons, inRegion: region)
                break
            }
        }
    }
    
    //リージョン観測のエラー時に呼ばれる
    func locationManager(manager: CLLocationManager,
        monitoringDidFailForRegion region: CLRegion?,
        withError error: NSError) {
        stopMonitoring()
        _delegate.onBeaconError(error.description)
    }
    
    //距離測定のエラー時に呼ばれる
    func locationManager(manager: CLLocationManager,
        didFailWithError error: NSError) {
        stopMonitoring()
        _delegate.onBeaconError(error.description)
    }
}

