import UIKit
import CoreLocation
import CoreBluetooth

//リージョン(15)
class BeaconRegion: NSObject {
    //システム
    var _locationManager: CLLocationManager! //ロケーションマネージャ
    var _delegate: BeaconRegionDelegate!     //デリゲート
    var _region: CLBeaconRegion!             //リージョン
    
    //情報
    var _uuid = ""               //UUID
    var _identifier = ""         //ID
    var _beacons = [CLBeacon]()  //ビーコン群
    var _beaconsW = [CLBeacon]() //ビーコン群(ワーク)
    
    
//====================
//初期化
//====================
    //初期化(16)
    init(locationManager: CLLocationManager?,
        delegate: BeaconRegionDelegate?, uuid: String) {
        super.init()
        _locationManager = locationManager
        _delegate = delegate
        _uuid = uuid
        _identifier = uuid
            
        //リージョンの生成
        _region = CLBeaconRegion(proximityUUID: NSUUID(UUIDString: uuid)!,
            identifier: _identifier)
        _region.notifyOnEntry = true
        _region.notifyOnExit = true
        _region.notifyEntryStateOnDisplay = false
    }
    
    //リージョン観測の開始(16)
    func startMonitoring() {
        _locationManager.startMonitoringForRegion(_region)
    }
    
    //リージョン観測の停止(17)
    func stopMonitoring() {
        _locationManager.stopMonitoringForRegion(_region)
    }
    
    
//====================
//CLLocationManagerDelegateの処理(18)
//====================
    //リージョン観測の開始時に呼ばれる
    func locationManager(manager: CLLocationManager,
        didStartMonitoringForRegion region: CLRegion) {
        
        //リージョン状態の要求
        _locationManager.requestStateForRegion(_region)
    }
    
    //リージョン状態の通知時に呼ばれる
    func locationManager(manager: CLLocationManager,
        didDetermineState state: CLRegionState, forRegion region: CLRegion) {
        if  state == CLRegionState.Inside {
            //レイジング観測の開始(19)
            _locationManager.startRangingBeaconsInRegion(_region)
        }
    }
    
    //リージョン外からリージョン内に移動した時に呼ばれる
    //(開始時にリージョン内にいると呼ばれない)
    func locationManager(manager: CLLocationManager,
        didEnterRegion region: CLRegion) {
        //レイジング観測の開始(19)
        _locationManager.startRangingBeaconsInRegion(_region)
    }
    
    //リージョン内からリージョン外に移動した時に呼ばれる
    //(20秒〜1分かかる)
    func locationManager(manager: CLLocationManager,
        didExitRegion region: CLRegion) {
        //レイジング観測の停止(20)
        _locationManager.stopRangingBeaconsInRegion(_region)
    }
    
    //リージョン内で距離が変化した時に呼ばれる
    func locationManager(manager: CLLocationManager,
        didRangeBeacons beacons: [CLBeacon],
        inRegion region: CLBeaconRegion) {
        //Enter(beaconsにあるが_beaconsにない)
        for i in 0..<beacons.count {
            let b = beacons[i]
            if !isExists(_beacons, beacon: b) {
                _delegate.onEnterRoom(self, beacon: b)
            }
        }
            
        //Exit(_beaconsにあるがbeaconsにない)
        for i in 0..<_beacons.count {
            let b = _beacons[i] as CLBeacon
            if !isExists(beacons, beacon: b) {
                _delegate.onExitRoom(self, beacon: b)
            }
        }
            
        //beaconsの更新
        _beacons.removeAll()
        for i in 0..<beacons.count {
            let beacon = beacons[i]
            if beacon.proximity != CLProximity.Unknown {
                _beacons.append(beacon)
            }
        }
        for i in 0..<_beacons.count {
            let beacon = beacons[i]
            if beacon.proximity == CLProximity.Unknown {
                _beacons.append(beacon)
            }
        }
            
        //更新
        _delegate.onUpdateBeacons(self, beacons:_beacons)
    }
    
    //Beacon存在チェック
    func isExists(beacons: [CLBeacon], beacon: CLBeacon) -> Bool {
        for (b) in beacons {
            if b.major == beacon.major && b.minor == beacon.minor {return true}
        }
        return false
    }
}


//====================
//BeaconRegionDelegateの定義(21)
//====================
protocol BeaconRegionDelegate {
    //リージョン外からリージョン内に移動した時に呼ばれる
    func onEnterRoom(sender: BeaconRegion, beacon: CLBeacon)
    
    //リージョン内からリージョン外に移動した時に呼ばれる
    func onExitRoom(sender: BeaconRegion, beacon: CLBeacon)
    
    //ビーコン情報の更新時に呼ばれる
    func onUpdateBeacons(sender: BeaconRegion, beacons: [CLBeacon])
    
    //エラー時に呼ばれる
    func onBeaconError(message: String)
}
